@@ -7,10 +7,10 @@
 uint32_t
 wob_color_to_argb(const struct wob_color color)
 {
-	uint8_t alpha = color.alpha * UINT8_MAX;
-	uint8_t red = color.red * UINT8_MAX;
-	uint8_t green = color.green * UINT8_MAX;
-	uint8_t blue = color.blue * UINT8_MAX;
+	uint8_t alpha = (uint8_t) (color.a * UINT8_MAX);
+	uint8_t red = (uint8_t) (color.r * UINT8_MAX);
+	uint8_t green = (uint8_t) (color.g * UINT8_MAX);
+	uint8_t blue = (uint8_t) (color.b * UINT8_MAX);
 
 	return (alpha << 24) + (red << 16) + (green << 8) + blue;
 }
@@ -19,10 +19,10 @@ struct wob_color
 wob_color_premultiply_alpha(const struct wob_color color)
 {
 	struct wob_color premultiplied_color = {
-		.alpha = color.alpha,
-		.red = color.red * color.alpha,
-		.green = color.green * color.alpha,
-		.blue = color.blue * color.alpha,
+		.a = color.a,
+		.r = color.r * color.a,
+		.g = color.g * color.a,
+		.b = color.b * color.a,
 	};
 
 	return premultiplied_color;
