@@ -16,7 +16,7 @@
 #define STR(x) #x
 
 // sizeof already includes NULL byte
-#define INPUT_BUFFER_LENGTH (3 * sizeof(unsigned long) + sizeof(" #FF000000 #FFFFFFFF #FFFFFFFF\n"))
+#define INPUT_BUFFER_LENGTH (3 * sizeof(unsigned long) + sizeof(" #000000FF #FFFFFFFF #FFFFFFFF\n"))
 
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
 
@@ -41,6 +41,12 @@
 #include "wlr-layer-shell-unstable-v1-client-protocol.h"
 #include "xdg-output-unstable-v1-client-protocol.h"
 
+enum wob_overflow_mode {
+	OVERFLOW_MODE_NONE,
+	OVERFLOW_MODE_WRAP,
+	OVERFLOW_MODE_NOWRAP,
+};
+
 struct wob_geom {
 	unsigned long width;
 	unsigned long height;
@@ -490,24 +496,28 @@ main(int argc, char **argv)
 	const char *usage =
 		"Usage: wob [options]\n"
 		"\n"
-		"  -h, --help                 Show help message and quit.\n"
-		"  --version                  Show the version number and quit.\n"
-		"  -v                         Increase verbosity of messages, defaults to errors and warnings only\n"
-		"  -t, --timeout <ms>         Hide wob after <ms> milliseconds, defaults to " STR(WOB_DEFAULT_TIMEOUT) ".\n"
-		"  -m, --max <%>              Define the maximum percentage, defaults to " STR(WOB_DEFAULT_MAXIMUM) ". \n"
-		"  -W, --width <px>           Define bar width in pixels, defaults to " STR(WOB_DEFAULT_WIDTH) ". \n"
-		"  -H, --height <px>          Define bar height in pixels, defaults to " STR(WOB_DEFAULT_HEIGHT) ". \n"
-		"  -o, --offset <px>          Define border offset in pixels, defaults to " STR(WOB_DEFAULT_BORDER_OFFSET) ". \n"
-		"  -b, --border <px>          Define border size in pixels, defaults to " STR(WOB_DEFAULT_BORDER_SIZE) ". \n"
-		"  -p, --padding <px>         Define bar padding in pixels, defaults to " STR(WOB_DEFAULT_BAR_PADDING) ". \n"
-		"  -a, --anchor <s>           Define anchor point; one of 'top', 'left', 'right', 'bottom', 'center' (default). \n"
-		"                             May be specified multiple times. \n"
-		"  -M, --margin <px>          Define anchor margin in pixels, defaults to " STR(WOB_DEFAULT_MARGIN) ". \n"
-		"  -O, --output <name>        Define output to show bar on or '*' for all. If ommited, focused output is chosen.\n"
-		"                             May be specified multiple times.\n"
-		"  --border-color <#argb>     Define border color\n"
-		"  --background-color <#argb> Define background color\n"
-		"  --bar-color <#argb>        Define bar color\n"
+		"  -h, --help                          Show help message and quit.\n"
+		"  --version                           Show the version number and quit.\n"
+		"  -v                                  Increase verbosity of messages, defaults to errors and warnings only\n"
+		"  -t, --timeout <ms>                  Hide wob after <ms> milliseconds, defaults to " STR(WOB_DEFAULT_TIMEOUT) ".\n"
+		"  -m, --max <%>                       Define the maximum percentage, defaults to " STR(WOB_DEFAULT_MAXIMUM) ". \n"
+		"  -W, --width <px>                    Define bar width in pixels, defaults to " STR(WOB_DEFAULT_WIDTH) ". \n"
+		"  -H, --height <px>                   Define bar height in pixels, defaults to " STR(WOB_DEFAULT_HEIGHT) ". \n"
+		"  -o, --offset <px>                   Define border offset in pixels, defaults to " STR(WOB_DEFAULT_BORDER_OFFSET) ". \n"
+		"  -b, --border <px>                   Define border size in pixels, defaults to " STR(WOB_DEFAULT_BORDER_SIZE) ". \n"
+		"  -p, --padding <px>                  Define bar padding in pixels, defaults to " STR(WOB_DEFAULT_BAR_PADDING) ". \n"
+		"  -a, --anchor <s>                    Define anchor point; one of 'top', 'left', 'right', 'bottom', 'center' (default). \n"
+		"                                      May be specified multiple times. \n"
+		"  -M, --margin <px>                   Define anchor margin in pixels, defaults to " STR(WOB_DEFAULT_MARGIN) ". \n"
+		"  -O, --output <name>                 Define output to show bar on or '*' for all. If ommited, focused output is chosen.\n"
+		"                                      May be specified multiple times.\n"
+		"  --border-color <#rgba>              Define border color\n"
+		"  --background-color <#rgba>          Define background color\n"
+		"  --bar-color <#rgba>                 Define bar color\n"
+		"  --overflow-mode <mode>              Change the overflow behavior. Valid options are `none`, `wrap` (default), and `nowrap`.\n"
+		"  --overflow-bar-color <#rgba>        Define bar color when overflowed\n"
+		"  --overflow-border-color <#rgba>     Define the border color when overflowed\n"
+		"  --overflow-background-color <#rgba> Define the background color when overflowed\n"
 		"\n";
 
 	struct wob app = {0};
@@ -515,6 +525,7 @@ main(int argc, char **argv)
 
 	unsigned long maximum = WOB_DEFAULT_MAXIMUM;
 	unsigned long timeout_msec = WOB_DEFAULT_TIMEOUT;
+	enum wob_overflow_mode overflow_mode = OVERFLOW_MODE_WRAP;
 	struct wob_geom geom = {
 		.width = WOB_DEFAULT_WIDTH,
 		.height = WOB_DEFAULT_HEIGHT,
@@ -524,29 +535,17 @@ main(int argc, char **argv)
 		.anchor = WOB_DEFAULT_ANCHOR,
 		.margin = WOB_DEFAULT_MARGIN,
 	};
+
 	struct wob_colors colors = {
-		.background =
-			(struct wob_color){
-				.alpha = 1.0,
-				.red = 0.0,
-				.green = 0.0,
-				.blue = 0.0,
-			},
-		.bar =
-			(struct wob_color){
-				.alpha = 1.0,
-				.red = 1.0,
-				.green = 1.0,
-				.blue = 1.0,
-			},
-		.border =
-			(struct wob_color){
-				.alpha = 1.0,
-				.red = 1.0,
-				.green = 1.0,
-				.blue = 1.0,
-			},
-	};
+		.background = (struct wob_color){.a = 1.0f, .r = 0.0f, .g = 0.0f, .b = 0.0f},
+		.bar = (struct wob_color){.a = 1.0f, .r = 1.0f, .g = 1.0f, .b = 1.0f},
+		.border = (struct wob_color){.a = 1.0f, .r = 1.0f, .g = 1.0f, .b = 1.0f}};
+
+	struct wob_colors overflow_colors = {
+		.background = (struct wob_color){.a = 1.0f, .r = 0.0f, .g = 0.0f, .b = 0.0f},
+		.bar = (struct wob_color){.a = 1.0f, .r = 1.0f, .g = 0.0f, .b = 0.0f},
+		.border = (struct wob_color){.a = 1.0f, .r = 1.0f, .g = 1.0f, .b = 1.0f}};
+
 	bool pledge = true;
 
 	char *disable_pledge_env = getenv("WOB_DISABLE_PLEDGE");
@@ -575,8 +574,12 @@ main(int argc, char **argv)
 		{"background-color", required_argument, NULL, 2},
 		{"bar-color", required_argument, NULL, 3},
 		{"verbose", no_argument, NULL, 'v'},
-	};
-	while ((c = getopt_long(argc, argv, "t:m:W:H:o:b:p:a:M:O:vh", long_options, &option_index)) != -1) {
+		{"overflow-mode", required_argument, NULL, 6},
+		{"overflow-bar-color", required_argument, NULL, 5},
+		{"overflow-background-color", required_argument, NULL, 7},
+		{"overflow-border-color", required_argument, NULL, 8}};
+
+	while ((c = getopt_long(argc, argv, "t:m:W:H:o:b:p:a:M:O:vh:f", long_options, &option_index)) != -1) {
 		switch (c) {
 			case 1:
 				if (!wob_parse_color(optarg, &strtoul_end, &(colors.border))) {
@@ -695,6 +698,39 @@ main(int argc, char **argv)
 			case 'v':
 				wob_log_inc_verbosity();
 				break;
+			case 5:
+				if (!wob_parse_color(optarg, &strtoul_end, &(overflow_colors.bar))) {
+					wob_log_error("Overflow bar color must be a value between #00000000 and #FFFFFFFF.");
+					return EXIT_FAILURE;
+				}
+				break;
+			case 6:
+				if (strcmp(optarg, "none") == 0) {
+					overflow_mode = OVERFLOW_MODE_NONE;
+				}
+				else if (strcmp(optarg, "wrap") == 0) {
+					overflow_mode = OVERFLOW_MODE_WRAP; // this is the default
+				}
+				else if (strcmp(optarg, "nowrap") == 0) {
+					overflow_mode = OVERFLOW_MODE_NOWRAP;
+				}
+				else {
+					wob_log_error("Invalid argument for overflow-mode. Valid options are none, wrap, and nowrap.");
+					return EXIT_FAILURE;
+				}
+				break;
+			case 7:
+				if (!wob_parse_color(optarg, &strtoul_end, &(overflow_colors.background))) {
+					wob_log_error("Overflow background color must be a value between #00000000 and #FFFFFFFF.");
+					return EXIT_FAILURE;
+				}
+				break;
+			case 8:
+				if (!wob_parse_color(optarg, &strtoul_end, &(overflow_colors.border))) {
+					wob_log_error("Overflow border color must be a value between #00000000 and #FFFFFFFF.");
+					return EXIT_FAILURE;
+				}
+				break;
 			default:
 				fprintf(stderr, "%s", usage);
 				return EXIT_FAILURE;
@@ -739,6 +775,7 @@ main(int argc, char **argv)
 	}
 
 	struct wob_colors old_colors;
+	struct wob_colors effective_colors = colors;
 
 	// Draw these at least once
 	wob_draw_background(app.wob_geom, argb, colors.background);
@@ -810,7 +847,6 @@ main(int argc, char **argv)
 						return EXIT_FAILURE;
 					}
 
-					old_colors = colors;
 					if (!wob_parse_input(input_buffer, &percentage, &colors.background, &colors.border, &colors.bar)) {
 						wob_log_error("Received invalid input");
 						if (!hidden) wob_hide(&app);
@@ -819,34 +855,54 @@ main(int argc, char **argv)
 						return EXIT_FAILURE;
 					}
 
+					old_colors = effective_colors;
 					if (percentage > maximum) {
-						wob_log_error("Received value %ld is above defined maximum %ld", percentage, maximum);
-						if (!hidden) wob_hide(&app);
-						wob_destroy(&app);
-
-						return EXIT_FAILURE;
+						switch (overflow_mode) {
+							case OVERFLOW_MODE_NONE:
+								wob_log_error("Received value %ld is above defined maximum %ld", percentage, maximum);
+								if (!hidden) wob_hide(&app);
+								wob_destroy(&app);
+								return EXIT_FAILURE;
+							case OVERFLOW_MODE_WRAP:
+								effective_colors = overflow_colors;
+								percentage %= maximum;
+								break;
+							case OVERFLOW_MODE_NOWRAP:
+								effective_colors = overflow_colors;
+								percentage = maximum;
+								break;
+						}
+					}
+					else {
+						effective_colors = colors;
 					}
 
-					wob_log_info("Received input { value = %ld, bg = %#x, border = %#x, bar = %#x }", percentage, colors.background, colors.border, colors.bar);
+					wob_log_info(
+						"Received input { value = %ld, bg = %#x, border = %#x, bar = %#x, overflow = %s }",
+						percentage,
+						effective_colors.background,
+						effective_colors.border,
+						effective_colors.bar,
+						overflow_mode == OVERFLOW_MODE_NONE ? "false" : "true"); // how should this be handled w/ the overflow colors?
 
 					if (hidden) {
 						wob_show(&app);
 					}
 
 					bool redraw_background_and_border = false;
-					if (wob_color_to_argb(old_colors.background) != wob_color_to_argb(colors.background)) {
+					if (wob_color_to_argb(old_colors.background) != wob_color_to_argb(effective_colors.background)) {
 						redraw_background_and_border = true;
 					}
-					else if (wob_color_to_argb(old_colors.border) != wob_color_to_argb(colors.border)) {
+					else if (wob_color_to_argb(old_colors.border) != wob_color_to_argb(effective_colors.border)) {
 						redraw_background_and_border = true;
 					}
 
 					if (redraw_background_and_border) {
-						wob_draw_background(app.wob_geom, argb, colors.background);
-						wob_draw_border(app.wob_geom, argb, colors.border);
+						wob_draw_background(app.wob_geom, argb, effective_colors.background);
+						wob_draw_border(app.wob_geom, argb, effective_colors.border);
 					}
 
-					wob_draw_percentage(app.wob_geom, argb, colors.bar, colors.background, percentage, maximum);
+					wob_draw_percentage(app.wob_geom, argb, effective_colors.bar, effective_colors.background, percentage, maximum);
 
 					wob_flush(&app);
 					hidden = false;
